# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2018-02-05 13:49
from __future__ import unicode_literals

import django.contrib.gis.db.models.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('chaos', '0005_auto_20180119_1257'),
    ]

    operations = [
        migrations.AddField(
            model_name='enterprise',
            name='geometry',
            field=django.contrib.gis.db.models.fields.MultiPolygonField(null=True, srid=4326, verbose_name='Geometry'),
        ),
        migrations.AddField(
            model_name='enterprise',
            name='geometry_900913',
            field=django.contrib.gis.db.models.fields.MultiPolygonField(null=True, srid=900913, verbose_name='Geometry 900913'),
        ),
        migrations.AddField(
            model_name='project',
            name='geometry',
            field=django.contrib.gis.db.models.fields.MultiPolygonField(null=True, srid=4326, verbose_name='Geometry'),
        ),
        migrations.AddField(
            model_name='project',
            name='geometry_900913',
            field=django.contrib.gis.db.models.fields.MultiPolygonField(null=True, srid=900913, verbose_name='Geometry 900913'),
        ),
        migrations.AddField(
            model_name='task',
            name='geometry',
            field=django.contrib.gis.db.models.fields.MultiPolygonField(null=True, srid=4326, verbose_name='Geometry'),
        ),
        migrations.AddField(
            model_name='task',
            name='geometry_900913',
            field=django.contrib.gis.db.models.fields.MultiPolygonField(null=True, srid=900913, verbose_name='Geometry 900913'),
        ),
    ]
