# -*- coding: utf-8 -*-
# Generated by Django 1.11.6 on 2018-04-11 14:27
from __future__ import unicode_literals

import django.contrib.gis.db.models.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('chaos', '0006_auto_20180205_0749'),
    ]

    operations = [
        migrations.AddField(
            model_name='task',
            name='extent',
            field=django.contrib.gis.db.models.fields.PolygonField(null=True, srid=4326, verbose_name='Extent'),
        ),
        migrations.AddField(
            model_name='task',
            name='extent_900913',
            field=django.contrib.gis.db.models.fields.PolygonField(null=True, srid=900913, verbose_name='Geometry 900913'),
        ),
        migrations.AlterField(
            model_name='task',
            name='geometry',
            field=django.contrib.gis.db.models.fields.PointField(null=True, srid=4326, verbose_name='Geometry'),
        ),
        migrations.AlterField(
            model_name='task',
            name='geometry_900913',
            field=django.contrib.gis.db.models.fields.PointField(null=True, srid=900913, verbose_name='Geometry 900913'),
        ),
    ]
